"use strict";

exports.__esModule = true;
exports.createAtom = createAtom;
var _excluded = ["state", "actions"];
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
/**
 * import { createAtom } from 'tiny-atom/core'
 *
 * const state = { count: 1 }
 * const actions = { inc: (atom, payload) => {} }
 * const atom = createAtom({ state, actions })
 *
 * atom.observe(atom => console.log(atom.get()))
 *
 * atom.get() // { count: 1 }
 * atom.dispatch('increment') // action
 * atom.dispatch('increment', { by: 2 }) // action with payload
 */

function createAtom(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
    _ref$state = _ref.state,
    state = _ref$state === void 0 ? {} : _ref$state,
    _ref$actions = _ref.actions,
    actions = _ref$actions === void 0 ? {} : _ref$actions,
    options = _objectWithoutPropertiesLoose(_ref, _excluded);
  var actionSeq = 0;
  var listeners = [];
  var get = function get() {
    return state;
  };
  var set = createSet();
  var swap = createSwap();
  var dispatch = createDispatch();
  var _options$evolve = options.evolve,
    evolve = _options$evolve === void 0 ? defaultEvolve : _options$evolve,
    _options$bindActions = options.bindActions,
    bindActions = _options$bindActions === void 0 ? defaultBindActions : _options$bindActions,
    debug = options.debug;
  var atom = {
    get: get,
    set: set,
    swap: swap,
    dispatch: dispatch,
    observe: observe,
    fuse: fuse,
    actions: bindActions(dispatch, actions)
  };
  return atom;
  function defaultEvolve(atom, action, actions) {
    if (!actions[action.type]) throw new Error("Missing action: " + action.type);
    return actions[action.type](atom, action.payload);
  }
  function defaultBindActions(dispatch, actions) {
    return Object.keys(actions).reduce(function (boundActions, actionType) {
      boundActions[actionType] = function (payload) {
        return dispatch(actionType, payload);
      };
      return boundActions;
    }, {});
  }
  function insert(listeners, f, i) {
    if (typeof i !== 'undefined') {
      for (var j = 0; j < listeners.length; j++) {
        if (listeners[j].i > i) {
          listeners.splice(j, 0, {
            f: f,
            i: i
          });
          return;
        }
      }
    }
    listeners.push({
      f: f,
      i: i
    });
  }
  function observe(f, i) {
    insert(listeners, f, i);
    return function unobserve() {
      for (var j = 0; j < listeners.length; j++) {
        if (listeners[j].f === f) {
          listeners.splice(j, 1);
          break;
        }
      }
    };
  }
  function fuse(_temp2) {
    var _ref2 = _temp2 === void 0 ? {} : _temp2,
      state = _ref2.state,
      moreActions = _ref2.actions;
    if (moreActions) {
      Object.assign(actions, moreActions);
      atom.actions = bindActions(dispatch, actions);
    }
    if (state) set(state, {
      silent: true
    });
  }
  function createDispatch(sourceActions) {
    sourceActions = sourceActions || [];
    return function dispatch(type, payload) {
      var action = {
        seq: ++actionSeq,
        type: type
      };
      if (typeof payload !== 'undefined') action.payload = payload;
      if (debug) {
        report('action', action, sourceActions);
        var history = sourceActions.concat([action]);
        var _dispatch = createDispatch(history);
        var _set = createSet(history);
        var _swap = createSwap(history);
        var boundActions = bindActions(_dispatch, actions);
        var actionAtom = Object.assign({}, atom, {
          dispatch: _dispatch,
          actions: boundActions,
          set: _set,
          swap: _swap
        });
        return evolve(actionAtom, action, actions);
      } else {
        return evolve(atom, action, actions);
      }
    };
  }
  function createSet(sourceActions, _temp3) {
    var _ref3 = _temp3 === void 0 ? {} : _temp3,
      _ref3$swap = _ref3.swap,
      swap = _ref3$swap === void 0 ? false : _ref3$swap;
    sourceActions = sourceActions || [];
    return function set(update, options) {
      if (options === void 0) {
        options = {};
      }
      options = typeof options === 'string' ? {
        message: options
      } : options;
      update = typeof update === 'function' ? update(state) : update;
      var action = {
        payload: update
      };
      var prevState = state;
      state = swap ? action.payload : Object.assign({}, state, action.payload);
      if (debug) report('update', action, sourceActions, prevState, options);
      listeners.forEach(function (l) {
        return l.f(atom);
      });
    };
  }
  function createSwap(sourceActions) {
    return createSet(sourceActions, {
      swap: true
    });
  }
  function report(type, action, sourceActions, prevState, options) {
    var info = _extends({}, options, {
      type: type,
      action: action,
      sourceActions: sourceActions,
      atom: atom
    });
    if (prevState) info.prevState = prevState;
    typeof debug === 'function' ? debug(info) : debug.forEach(function (debug) {
      return debug(info);
    });
  }
}