"use strict";

exports.__esModule = true;
exports.createLog = void 0;
var _deepDiff = _interopRequireDefault(require("deep-diff"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
var dictionary = {
  N: {
    color: '#4CAF50',
    text: 'ADDED',
    atext: 'added'
  },
  E: {
    color: '#2196F3',
    text: 'UPDATED'
  },
  D: {
    color: '#F44336',
    text: 'DELETED',
    atext: 'deleted'
  },
  A: {
    color: '#2196F3',
    text: 'ARRAY'
  }
};
var createLog = exports.createLog = function createLog(options) {
  if (options === void 0) {
    options = {};
  }
  options.diff = typeof options.diff === 'undefined' ? true : options.diff;
  options.actions = typeof options.actions === 'undefined' ? false : options.actions;
  options.updates = typeof options.updates === 'undefined' ? true : options.updates;
  options.include = options.include || [];
  options.exclude = options.exclude || [];
  options.level = options.level || 'loud';
  var logger = options.logger || console;
  var tryCatch = function tryCatch(fn, elseFn) {
    try {
      fn();
    } catch (e) {
      elseFn();
    }
  };
  var log = function log() {
    return logger.log.apply(logger, arguments);
  };
  var groupStart = function groupStart() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    return tryCatch(function () {
      return logger.groupCollapsed.apply(logger, args);
    }, function () {
      return logger.log.apply(logger, args);
    });
  };
  var groupEnd = function groupEnd() {
    return tryCatch(function () {
      return logger.groupEnd();
    }, function () {});
  };
  return function (_ref) {
    var type = _ref.type,
      atom = _ref.atom,
      action = _ref.action,
      sourceActions = _ref.sourceActions,
      prevState = _ref.prevState,
      message = _ref.message,
      silent = _ref.silent;
    if (options.level === 'none') return;
    var sourceAction = type === 'action' ? action : sourceActions[sourceActions.length - 1] || {};
    if (options.include.length && !options.include.includes(sourceAction.type)) return;
    if (options.exclude.length && options.exclude.includes(sourceAction.type)) return;
    if (silent && options.level !== 'verbose') return;
    if (type === 'action' && options.actions) {
      var actions = sourceActions.concat(action);
      groupStart("\uD83D\uDE80 " + actions.map(function (a) {
        return a.type;
      }).join(' → '));
      log('payload', action.payload);
      log('chain', actions);
      groupEnd();
    }
    if (type === 'update' && options.updates) {
      var diff;
      var diffSummaryMap = {};
      var diffSummary = [];
      if (options.diff) {
        diff = (0, _deepDiff.default)(prevState, atom.get());
        if (diff) {
          diff.forEach(function (change) {
            diffSummaryMap[change.kind] = diffSummaryMap[change.kind] ? diffSummaryMap[change.kind] + 1 : 1;
          });
          Object.keys(dictionary).forEach(function (kind) {
            if (diffSummaryMap[kind]) {
              diffSummary.push(diffSummaryMap[kind] + " " + dictionary[kind].text.toLowerCase());
            }
          });
        }
      }
      groupStart("\uD83D\uDE4C " + (sourceActions.map(function (a) {
        return a.type;
      }).join(' → ') || '–'), message ? '»' : '', message || '');
      log('curr', atom.get());
      log('prev', prevState);
      log('update', action.payload);
      log.apply(void 0, ['chain'].concat(sourceActions));
      if (diff) printDiff(diff);
      groupEnd();
    }
  };
  function style(kind) {
    return "color: " + dictionary[kind].color + "; font-weight: bold;";
  }
  function render(diff) {
    var kind = diff.kind,
      path = diff.path,
      lhs = diff.lhs,
      rhs = diff.rhs,
      index = diff.index,
      item = diff.item;
    path = path || [];
    switch (kind) {
      case 'E':
        return [path.join('.'), lhs, '→', rhs];
      case 'N':
        return [path.join('.'), rhs];
      case 'D':
        return [path.join('.')];
      case 'A':
        return [path.join('.') + "[" + index + "]", dictionary[item.kind].atext].concat(render(item).slice(1));
      default:
        return [];
    }
  }
  function printDiff(diff) {
    diff.forEach(function (elem) {
      var kind = elem.kind;
      logger.log.apply(logger, ["%c" + dictionary[kind].text, style(kind)].concat(render(elem)));
    });
  }
};